package cn.rankin.apiweb.controller;

import cn.rankin.apiweb.assist.resolver.NeedUser;
import cn.rankin.apiweb.service.collection.CollectionService;
import cn.rankin.apiweb.service.user.UserService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.app.vo.DeviceUserVo;
import cn.rankin.data.api.app.vo.ItemVo;
import cn.rankin.data.api.user.dto.CollectionDTO;
import cn.rankin.data.api.user.entity.UserTag;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping(value = "/user/collection")
public class CollectionController {

    @Autowired
    private CollectionService collectionService;

    @Autowired
    private UserService userService;

    @RequestMapping(method = RequestMethod.POST)
    public APIResult<Boolean> put(@NeedUser DeviceUserVo user, @RequestBody CollectionDTO collectionDTO) {
        return collectionService.put(user.getUid(), user.getMerchantId(), collectionDTO.getPid());
    }

    @RequestMapping(value = "/{productId}", method = RequestMethod.DELETE)
    public APIResult<Boolean> delete(@NeedUser DeviceUserVo user, @PathVariable("productId") String productId) {
        return collectionService.delete(user.getUid(), productId);
    }

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<Page<ItemVo>> getPage(@NeedUser DeviceUserVo user, @RequestParam(value = "tagId", required = false) String tagId,
                                           @RequestParam(value = "pageNo", defaultValue = "0") Integer pageNo,
                                           @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize) {
        String userId = user.getUid();
        String merchantId = user.getMerchantId();
        if (StringUtils.isEmpty(tagId)) {
            return collectionService.getUserCollection(userId, merchantId, pageNo, pageSize);
        }else {
            //tagId   可能为渠道标签ID，也可能为用户标签ID，先做判断再去获取数据
            UserTag userTag = userService.findUserTagById(tagId);
            if(null != userTag){
                //用户标签，
                return collectionService.getUserConllectionByUserTag(userId, merchantId, tagId, pageNo, pageSize);
            }

            //渠道标签
            return collectionService.getUserCollection(userId, merchantId, tagId, pageNo, pageSize);
        }
    }
}
