package cn.rankin.apiweb.service.product;

import cn.rankin.apiweb.code.ApiWebCode;
import cn.rankin.apiweb.service.resource.ResourceService;
import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.model.BaseCode;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.data.api.app.dto.ItemSearchDTO;
import cn.rankin.data.api.app.vo.*;
import cn.rankin.data.api.product.entity.*;
import cn.rankin.data.api.product.vo.CourseItemVo;
import cn.rankin.data.api.product.vo.SupportItemVo;
import cn.rankin.data.api.product.vo.TrainingItemVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static cn.rankin.apiweb.utils.DTOConverter.convert;

@Service
public class ProductService {

    @Autowired
    private ProductClient productClient;

    @Autowired
    private ResourceService resourceService;

    public APIResult<List<RecommendVo>> getRecommendCourses(String merchantId) {
        APIResult<List<Recommend>> apiResult = productClient.getRecommendCourses(merchantId);
        if (!apiResult.getSuccess()) {
            return APIResult.error(new BaseCode(apiResult.getCode(), apiResult.getMessage()));
        }

        List<Recommend> recommendList = apiResult.getData();
        List<RecommendVo> recommendVoList = new ArrayList<>();
        for (Recommend recommend : recommendList) {
            RecommendVo recommendVo = convert(recommend);
            recommendVoList.add(recommendVo);
        }
        return APIResult.ok(recommendVoList);
    }

    public APIResult<MerchantProduct> getMerchantProduct(String productId, String merchantId) {
        return productClient.getMerchantProduct(productId, merchantId);
    }

    public APIResult<List<ItemVo>> findItemByIds(String merchantId, List<String> productIdList) {
        return productClient.findItemByIds(merchantId, productIdList);
    }

    public APIResult<Page<ItemVo>> findItemPageByIds(ItemSearchDTO itemSearchDTO) {
        return productClient.findItemPageByIds(itemSearchDTO);
    }

    public APIResult<CourseVo> getCourseLessons(String courseId) {
        APIResult<Course> courseAPIResult = productClient.getCourseLessons(courseId);
        if (!courseAPIResult.getSuccess()) {
            return APIResult.error(new BaseCode(courseAPIResult.getCode(), courseAPIResult.getMessage()));
        }
        Course course = courseAPIResult.getData();
        CourseVo courseVo = convert(course);
        return APIResult.ok(courseVo);
    }

    public APIResult<CourseDetailVo> getCourseDetail(String courseId, String merchantId) {
        APIResult<CourseItemVo> apiResult = productClient.getCourseDetail(courseId, merchantId);
        if (!apiResult.getSuccess()) {
            return APIResult.error(new BaseCode(apiResult.getCode(), apiResult.getMessage()));
        }
        CourseItemVo courseItemVo = apiResult.getData();
        CourseDetailVo courseDetailVo = convert(courseItemVo);
        return APIResult.ok(courseDetailVo);
    }

    public APIResult<List<ItemVo>> getSupportByCourseId(String courseId, String merchantId) {
        return productClient.getSupportByCourseId(courseId, merchantId);
    }

    public APIResult<SupportItemVo> getSupport(String supportId, String merchantId) {
        return productClient.getSupport(supportId, merchantId);
    }

    public APIResult<TrainingItemVo> getTraining(String id, String merchantId) {
        return productClient.getTraining(id, merchantId);
    }

    public APIResult<LessonVo> getLesson(String lessonId, String courseId, Boolean auth) {
        APIResult<Lesson> apiResult = productClient.getLesson(lessonId, courseId);
        if (!apiResult.getSuccess()) {
            return APIResult.error(new BaseCode(apiResult.getCode(), apiResult.getMessage()));
        }
        Lesson lesson = apiResult.getData();
        if (lesson.getSort() == 0 || Boolean.TRUE.equals(auth)) {
            LessonVo lessonVo = resourceService.setWareUrl(lesson);
            return APIResult.ok(lessonVo);

        }
        return APIResult.error(ApiWebCode.ACCESS_DENIED);
    }

    public APIResult<Lesson> getLesson(String lessonId, String courseId) {
        APIResult<Lesson> apiResult = productClient.getLesson(lessonId, courseId);
        if (!apiResult.getSuccess()) {
            return APIResult.error(new BaseCode(apiResult.getCode(), apiResult.getMessage()));
        }
        return apiResult;
    }

    public List<Poster> getPosters(String merchantId){
        return productClient.getPosters(merchantId, 0L, 10, "sort", Sort.Direction.ASC);
    }

    public Course getCourse(String id) {
        return productClient.getCourse(id);
    }


    public List<ItemVo> findItemByPids(List<String> productIdList, String merchantId) {
        return productClient.findItemByPids(productIdList,merchantId);
    }

}
