package cn.rankin.productservice.controller.cms;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.product.dto.MerchantProductDTO;
import cn.rankin.data.api.product.dto.MerchantProductSearchDTO;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.data.api.product.entity.MerchantProduct;
import cn.rankin.productservice.code.ProductServiceAPICode;
import cn.rankin.productservice.service.MerchantProductService;
import cn.rankin.productservice.service.app.AppGoodsService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.LinkedHashMap;
import java.util.List;

@RestController
@RequestMapping(value = "/merchant/product")
public class MerchantProductController {

    @Autowired
    private MerchantProductService merchantProductService;

    @RequestMapping(method = RequestMethod.GET)
    public APIResult<Page<MerchantProduct>> search(MerchantProductSearchDTO searchDTO) {
        MerchantProduct merchantProduct = new MerchantProduct();

        String code = searchDTO.getCode();
        if (!StringUtils.isEmpty(code)) {
            merchantProduct.setCode("%" + code + "%");
        }

        String name = searchDTO.getName();
        if (!StringUtils.isEmpty(name)) {
            merchantProduct.setName("%" + name + "%");
        }

        // others
        merchantProduct.setType(searchDTO.getType());
        merchantProduct.setMerchantId(searchDTO.getMerchantId());

        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap<>();
        sort.put("gmtModified", BaseOrderEnum.DESC);

        return merchantProductService.search(merchantProduct, searchDTO.getPageNo(), searchDTO.getPageSize(), sort);
    }

    @RequestMapping(value = "/detail", method = RequestMethod.GET)
    public APIResult<MerchantProduct> getProduct(@RequestParam("pid") String productId, @RequestParam("merchantId") String merchantId) {
        MerchantProduct merchantProduct = merchantProductService.getProduct(productId, merchantId);
        if (merchantProduct == null) {
            return APIResult.error(ProductServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(merchantProduct);
    }

    @RequestMapping(value = "/status", method = RequestMethod.PUT)
    public APIResult<Boolean> change(@RequestBody MerchantProductDTO merchantProductDTO) {
        String productId = merchantProductDTO.getPid();
        String merchantId = merchantProductDTO.getMerchantId();
        BaseStatusEnum status = merchantProductDTO.getStatus();
        if (StringUtils.isEmpty(productId) || StringUtils.isEmpty(merchantId) || status == null) {
            return APIResult.error(ProductServiceAPICode.PARAMETER_ERROR);
        }
        return merchantProductService.change(productId, merchantId, status);
    }

    @RequestMapping(value = "/tags", method = RequestMethod.PUT)
    public APIResult<MerchantProduct> mountTags(@RequestBody MerchantProductDTO productDTO) {
        String productId = productDTO.getPid();
        if (StringUtils.isEmpty(productId)) {
            return APIResult.error(ProductServiceAPICode.PARAMETER_ERROR);
        }

        String merchantId = productDTO.getMerchantId();
        if (StringUtils.isEmpty(merchantId)) {
            return APIResult.error(ProductServiceAPICode.PARAMETER_ERROR);
        }

        List<String> tagIdList = productDTO.getTags();

        return merchantProductService.mountTags(productId, merchantId, tagIdList);
    }

    @RequestMapping(value = "/list2", method = RequestMethod.POST)
    public APIResult<List<MerchantProduct>> getList2(@RequestBody List<String> productIdList, @RequestParam("merchantId") String merchantId) {
        return merchantProductService.findByPidList_bak(productIdList, merchantId);
    }

    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public APIResult<List<MerchantProduct>> getList(@RequestBody List<String> productIdList, @RequestParam("merchantId") String merchantId) {
        List<MerchantProduct> merchantProductList = merchantProductService.findByPidList(productIdList, merchantId);
        return APIResult.ok(merchantProductList);
    }

    @RequestMapping(value = "/pids", method = RequestMethod.POST)
    public List<MerchantProduct> findByPidsAndMerchantId(@RequestBody List<String> productIdList, @RequestParam("merchantId") String merchantId){
        return merchantProductService.findByPidsAndMerchantId(productIdList, merchantId);
    }

    @RequestMapping(value = "/pid", method = RequestMethod.GET)
    public MerchantProduct findByPid(@RequestParam("pid") String pid){
        return merchantProductService.findByPid(pid);
    }



}
