package cn.rankin.userservice.controller;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.EventTypeEnum;
import cn.rankin.data.api.user.dto.DownloadInfoDTO;
import cn.rankin.data.api.user.entity.DownloadInfo;
import cn.rankin.userservice.service.DownloadInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(value = "/download")
public class DownloadInfoController {

    @Autowired
    private DownloadInfoService downloadInfoService;

    /**
     * 添加下载
     * @param dto
     * @return
     */
    @RequestMapping(method = RequestMethod.POST)
    public DownloadInfo add(@RequestBody DownloadInfoDTO dto) {
        //TODO
        //addEventLog();

        return downloadInfoService.add(dto);
    }

    /**
     * 下载列表
     * @param uid
     * @param mac
     * @return
     */
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public List<DownloadInfo> findByUserId(@RequestParam("uid") String uid,
                                           @RequestParam("mac") String mac){
        return downloadInfoService.findByUserId(uid,mac,EventTypeEnum.DOWNLOAD,BaseStatusEnum.NORMAL);
    }

    /**
     * 查询下载
     * @param uid
     * @param lessonId
     * @param mac
     * @return
     */
    @RequestMapping(value = "", method = RequestMethod.GET)
    public DownloadInfo findByLessonUserId(@RequestParam("uid") String uid,
                                           @RequestParam("lessonId") String lessonId,
                                           @RequestParam("mac") String mac){
        return downloadInfoService.findByLessonUserId(uid, lessonId, mac, EventTypeEnum.DOWNLOAD, BaseStatusEnum.NORMAL);
    }

    /**
     * 删除下载
     * @param uid
     * @param lessonId
     * @param mac
     * @return
     */
    @RequestMapping(value = "/delete", method = RequestMethod.DELETE)
    public DownloadInfo deleteByLessonUserId(@RequestParam("uid") String uid,
                                             @RequestParam("lessonId") String lessonId,
                                             @RequestParam("mac") String mac){
        return downloadInfoService.deleteByLessonUserId(uid, lessonId, mac);
    }

    /**
     * 更新下载内容
     * @param uid
     * @param lessonId
     * @param mac
     * @return
     */
    @RequestMapping(value = "/update/resource", method = RequestMethod.PUT)
    public DownloadInfo updateByLessonUserId(@RequestParam("uid") String uid,
                                             @RequestParam("lessonId") String lessonId,
                                             @RequestParam("mac") String mac){
        return downloadInfoService.updateByLessonUserId(uid, lessonId, mac);
    }

    @RequestMapping(value = "/fail", method = RequestMethod.PUT)
    public boolean updateDownloadFail(@RequestParam("uid") String uid,
                                @RequestParam("mac") String mac){
        return downloadInfoService.updateDownloadFail(uid, mac);
    }

    @RequestMapping(value = "/update", method = RequestMethod.PUT)
    public DownloadInfo update(@RequestBody DownloadInfo downloadInfo){
        return downloadInfoService.update(downloadInfo);
    }

    @RequestMapping(value = "/set/update/{lessonId}", method = RequestMethod.PUT)
    public int setUpdateByLessonId(@PathVariable("lessonId") String lessonId){
        return downloadInfoService.setUpdateByLessonId(lessonId);
    }

    /**
     * 查询下载进度
     * @param uid
     * @param lessonId
     * @param mac
     * @return
     */
    @RequestMapping(value = "/status", method = RequestMethod.GET)
    public DownloadInfo findDownloadStatus(@RequestParam("uid") String uid,
                                           @RequestParam("lessonId") String lessonId,
                                           @RequestParam("mac") String mac){
        return downloadInfoService.findDownloadStatus(uid, lessonId, mac);
    }

}
