package cn.rankin.userservice.repository;

import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.EventTypeEnum;
import cn.rankin.common.utils.jpa.BasicJpaRepository;
import cn.rankin.data.api.user.entity.DownloadInfo;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;

import java.util.List;

public interface DownloadInfoRepository extends BasicJpaRepository<DownloadInfo, String> {

    @Query(value = "select info from DownloadInfo info where info.userId = ?1 AND info.mac = ?2 AND info.eventType= ?3 AND info.status = ?4 ORDER BY info.gmtCreated DESC ")
    List<DownloadInfo> findByUserId(String uid, String mac, EventTypeEnum eventType, BaseStatusEnum status);

    @Query(value = "select info from DownloadInfo info where info.userId = ?1 AND info.lessonId = ?2 AND info.mac = ?3 AND info.eventType = ?4  AND info.status = ?5 ORDER BY info.gmtCreated DESC ")
    DownloadInfo findByLessonUserId(String uid, String lessonId, String mac, EventTypeEnum eventType, BaseStatusEnum status);

    @Query(value = "select info from DownloadInfo info where info.userId = ?1 AND info.lessonId = ?2  AND info.mac = ?3 ORDER BY info.gmtCreated DESC ")
    DownloadInfo findByLessonUserId(String uid, String lessonId, String mac);

    @Modifying
    @Query(value = "update DownloadInfo info set info.status = ?4 where info.userId = ?1 and info.lessonId = ?2   AND info.mac = ?3 ")
    int updateByLessonUserId(String uid, String lessonId, String mac, BaseStatusEnum status);

    @Modifying
    @Query(value = "update DownloadInfo info set info.needUpdate = 1 where info.lessonId = ?1 AND info.status = ?2")
    int setUpdateByLessonId(String lessonId, BaseStatusEnum status);

    @Modifying
    @Query(value = "update DownloadInfo info set info.downloadStatus = ?3 where info.userId = ?1 AND info.mac = ?2 AND info.downloadStatus = 2")
    int updateDownloadFail(String uid, String mac, int failDownload);

    @Query(value = "select info from DownloadInfo info where info.userId = ?1 AND info.lessonId = ?2  AND info.mac = ?3 AND info.downloadStatus = 2 ORDER BY info.gmtCreated DESC ")
    DownloadInfo findDownloadStatus(String uid, String lessonId, String mac);
}
