package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.data.api.user.dto.CollectionDTO;
import cn.rankin.data.api.user.entity.Collection;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.repository.CollectionRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

import static cn.rankin.userservice.utils.DTOConverter.convert;

@Service
@Slf4j
public class CollectionService {

    @Autowired
    private CollectionRepository collectionRepository;

    public APIResult<Collection> getCollection(String userId, String productId) {
        Collection sample = new Collection();
        sample.setUserId(userId);
        sample.setPid(productId);
        sample.setStatus(BaseStatusEnum.NORMAL);
        Collection collection = collectionRepository.findFirst(sample);
        if (collection == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(collection);
    }

    @Transactional
    public APIResult<Boolean> put(CollectionDTO collectionDTO) {
        Collection collection = collectionRepository.findByPidAndUserId(collectionDTO.getPid(), collectionDTO.getUserId());
        if (collection != null) {
            if (collection.getStatus().equals(BaseStatusEnum.NORMAL)) {
                return APIResult.error(UserServiceAPICode.ALREADY_EXISTS);
            }else {
                collection.setStatus(BaseStatusEnum.NORMAL);
                collectionRepository.save(collection);
                return APIResult.ok(true);
            }
        }

        Collection newCollection = convert(collectionDTO);
        collectionRepository.save(newCollection);
        return APIResult.ok(true);
    }

    public APIResult<Page<Collection>> getPage(String userId, Integer pageNo, Integer pageSize) {
        Collection sample = new Collection();
        sample.setUserId(userId);

        Long total = collectionRepository.count(sample);
        Page page = new Page(total, pageNo, pageSize);
        if (total == 0) {
            return APIResult.ok(page);
        }

        List<Collection> list = collectionRepository.find(sample, page.getStart(), pageSize);
        page.setList(list);
        return APIResult.ok(page);
    }

    public APIResult<List<Collection>> gets(String userId, Integer size) {
        Collection sample = new Collection();
        sample.setUserId(userId);
        sample.setStatus(BaseStatusEnum.NORMAL);
        List<Collection> list = collectionRepository.find(sample, 0L, size);
        return APIResult.ok(list);
    }

    @Transactional
    public APIResult<Boolean> delete(String userId, String productId) {
        Integer count = collectionRepository.deleteByPidAndUserId(productId, userId);
        if (count > 0) {
            return APIResult.ok(true);
        }
        return APIResult.error(UserServiceAPICode.NOT_EXISTS);
    }
}
