package cn.rankin.userservice.service;

import cn.rankin.common.utils.constant.DownloadStatus;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.enums.EventTypeEnum;
import cn.rankin.data.api.user.dto.DownloadInfoDTO;
import cn.rankin.data.api.user.entity.DownloadInfo;
import cn.rankin.userservice.repository.DownloadInfoRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

import static cn.rankin.userservice.utils.DTOConverter.convert;

@Service
public class DownloadInfoService {

    @Autowired
    private DownloadInfoRepository downloadInfoRepository;

    /**
     * 增加下载信息，
     * 若已存在，更改状态
     * @param dto
     * @return
     */
    public DownloadInfo add(DownloadInfoDTO dto) {
        DownloadInfo entity = downloadInfoRepository.findByLessonUserId(dto.getUserId(), dto.getLessonId(), dto.getMac());
        if(entity != null){
            DownloadInfo downloadInfo = convert(dto);
            downloadInfo.setId(entity.getId());
            return downloadInfoRepository.update(downloadInfo);
        }else{
            DownloadInfo downloadInfo = convert(dto);
            return downloadInfoRepository.save(downloadInfo);
        }

    }

    /**
     * 用户所有下载列表    不包含已删除
     * @param uid
     * @param eventType
     * @param status
     * @return
     */
    public List<DownloadInfo> findByUserId(String uid, String mac, EventTypeEnum eventType, BaseStatusEnum status) {
        return downloadInfoRepository.findByUserId(uid, mac, eventType, status);
    }

    /**
     * 查询下载信息
     * 根据用户ID   课程ID
     * @param uid
     * @param lessonId
     * @param eventType
     * @param status
     * @return
     */
    public DownloadInfo findByLessonUserId(String uid, String lessonId, String mac, EventTypeEnum eventType, BaseStatusEnum status) {
        return downloadInfoRepository.findByLessonUserId(uid, lessonId, mac, eventType, status);
    }

    /**
     * 删除下载   （数据库逻辑删除）
     * @param uid
     * @param lessonId
     * @return
     */
    public DownloadInfo deleteByLessonUserId(String uid, String lessonId, String mac) {
        DownloadInfo entity = downloadInfoRepository.findByLessonUserId(uid, lessonId, mac);
        if(entity != null){
            entity.setStatus(BaseStatusEnum.DEL);
            return downloadInfoRepository.update(entity);
        }else{
            return null;
        }
    }

    /**
     * 更新下载
     * @param uid
     * @param lessonId
     * @return
     */
    public DownloadInfo updateByLessonUserId(String uid, String lessonId, String mac) {
        DownloadInfo entity = downloadInfoRepository.findByLessonUserId(uid, lessonId, mac);
        if(entity != null){
            entity.setNeedUpdate(0);
            return downloadInfoRepository.update(entity);
        }else{
            return null;
        }
    }

    public int setUpdateByLessonId(String lessonId) {
        return downloadInfoRepository.setUpdateByLessonId(lessonId,BaseStatusEnum.NORMAL);
    }

    public DownloadInfo update(DownloadInfo downloadInfo) {
        return downloadInfoRepository.update(downloadInfo);
    }

    @Transactional
    public boolean updateDownloadFail(String uid, String mac) {
        downloadInfoRepository.updateDownloadFail(uid, mac, DownloadStatus.FAIL_DOWNLOAD);
        return true;
    }

    public DownloadInfo findDownloadStatus(String uid, String lessonId, String mac) {
        return downloadInfoRepository.findDownloadStatus(uid, lessonId, mac);
    }
}
