package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.model.APIResult;
import cn.rankin.data.api.user.dto.TerminalDeviceDTO;
import cn.rankin.data.api.user.entity.TerminalDevice;
import cn.rankin.userservice.code.UserServiceAPICode;
import cn.rankin.userservice.repository.TerminalDeviceRepository;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

@Service
public class TerminalDeviceService {

    @Autowired
    private TerminalDeviceBindLogService terminalDeviceBindLogService;

    @Autowired
    private TerminalDeviceRepository terminalDeviceRepository;

    @Transactional
    public APIResult<TerminalDevice> bind(TerminalDeviceDTO terminalDeviceDTO) {
        String deviceCode = terminalDeviceDTO.getDeviceCode();
        String merchant = terminalDeviceDTO.getMerchant();
        String userId = terminalDeviceDTO.getUserId();
        String brand = terminalDeviceDTO.getBrand();
        String ip = terminalDeviceDTO.getIp();
        String modelNo = terminalDeviceDTO.getModelNo();
        String terminal = terminalDeviceDTO.getTerminal();

        TerminalDevice terminalDevice = terminalDeviceRepository.findByUserId(userId);
        if (terminalDevice != null) {
            if(!deviceCode.equals(terminalDevice.getDeviceCode())){
                //存在设备号不一致，账号被其他设备使用中
                return APIResult.error(UserServiceAPICode.USER_IS_BOUND);
            }else{
                //根据设备号，merchant,删除设备绑定记录,后面会重新添加绑定关系
                deleteTerminalDeviceByDeviceCodeMerchant(deviceCode, merchant);
                terminalDeviceBindLogService.insert(deviceCode, merchant, userId, brand, ip, modelNo, terminal+"_del");
            }
        }

        terminalDevice = new TerminalDevice();
        terminalDevice.setUserId(userId);
        terminalDevice.setDeviceCode(deviceCode);
        terminalDevice.setMerchant(merchant);
        terminalDevice.setBrand(brand);
        terminalDevice.setModelNo(modelNo);
        terminalDevice.setIp(ip);
        terminalDevice.setTerminal(terminal);

        TerminalDevice deviceBind = terminalDeviceRepository.save(terminalDevice);
        terminalDeviceBindLogService.insert(deviceCode, merchant, userId, brand, ip, modelNo, terminal);

        return APIResult.ok(deviceBind);
    }

    private void deleteTerminalDeviceByDeviceCodeMerchant(String deviceCode, String merchant) {
        terminalDeviceRepository.deleteByDeviceCodeAndMerchant(deviceCode, merchant);
        terminalDeviceRepository.deleteByDeviceCodeAndMerchant(deviceCode, null);
    }

    @Transactional
    public APIResult<Boolean> unbind(String userId) {
        TerminalDevice terminalDevice = terminalDeviceRepository.findByUserId(userId);
        if (terminalDevice == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        terminalDeviceRepository.deleteByUserId(userId);
        return APIResult.ok(Boolean.TRUE);
    }

/*    public APIResult<TerminalDevice> findByUserId(String userId) {
        TerminalDevice terminalDevice = terminalDeviceRepository.findByUserId(userId);
        if (terminalDevice == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }
        return APIResult.ok(terminalDevice);
    }*/

    public TerminalDevice findByUserId(String userId) {
        return terminalDeviceRepository.findByUserId(userId);
    }

    public List<TerminalDevice> findByDeviceCode(String deviceCode) {
        List<TerminalDevice> terminalDeviceList = terminalDeviceRepository.findByDeviceCode(deviceCode);
        /*if (terminalDevice == null) {
            return APIResult.error(UserServiceAPICode.NOT_EXISTS);
        }*/
        return terminalDeviceList;
    }

    public APIResult updatebind(TerminalDeviceDTO terminalDeviceDTO) {
        if(StringUtils.isBlank(terminalDeviceDTO.getId())){
            return APIResult.error(UserServiceAPICode.PARAMETER_ERROR);
        }

        TerminalDevice terminalDevice = terminalDeviceRepository.getOne(terminalDeviceDTO.getId());
        if(null == terminalDevice){
            return APIResult.error(UserServiceAPICode.PARAMETER_ERROR);
        }
        terminalDevice.setMerchant(terminalDeviceDTO.getMerchant());

        terminalDeviceRepository.update(terminalDevice);

        return APIResult.ok();
    }
}
