package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.common.utils.util.ListUtil;
import cn.rankin.data.api.user.entity.UserTagProductRelation;
import cn.rankin.userservice.repository.UserTagProductRelationRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

@Slf4j
@Service
public class UserTagProductRelationService {

    @Autowired
    private UserTagProductRelationRepository relationRepository;

    public List<UserTagProductRelation> findByUserTagId(String tagId) {
        UserTagProductRelation relation = new UserTagProductRelation();
        relation.setTagId(tagId);
        relation.setStatus(BaseStatusEnum.NORMAL);


        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap() { {
            this.put("sort", BaseOrderEnum.ASC);
        }};
        List<UserTagProductRelation> relationList = relationRepository.find(relation, JpaSortUtil.sort(sort));

        return relationList;
    }

    public List<UserTagProductRelation> findByTagId(String tagId, BaseStatusEnum status) {
        return relationRepository.findByTagId(tagId, status);
    }

    @Transactional
    public void updateByTagId(String tagId, List<String> productIdList) {
        if (productIdList == null) {
            return;
        }else if (productIdList.size() == 0) {
            relationRepository.deleteByTagId(tagId);
            return;
        }

        List<UserTagProductRelation> relationList = relationRepository.findByTagId(tagId);

        List<String> currentIdList = new ArrayList<>();
        relationList.forEach( relation -> {
            String productId = relation.getPid();
            if (productIdList.contains(productId)) {
                relation.setStatus(BaseStatusEnum.NORMAL);
            }else {
                relation.setStatus(BaseStatusEnum.DEL);
            }
            currentIdList.add(productId);
        });

        List<String> notExistItemIdList = ListUtil.subtract(productIdList, currentIdList);
        if (notExistItemIdList != null && notExistItemIdList.size() > 0) {
            notExistItemIdList.forEach( productId -> {
                UserTagProductRelation relation = new UserTagProductRelation();
                relation.setPid(productId);
                relation.setTagId(tagId);
                relation.setSort(0);
                relation.setStatus(BaseStatusEnum.NORMAL);
                relationList.add(relation);
            });
        }

        // 排序
        relationList.forEach( relation -> {
            String productId = relation.getPid();
            Integer index = productIdList.indexOf(productId);
            relation.setSort(index);
        });

        relationRepository.save(relationList);
    }

    public Page<UserTagProductRelation> findPageByUserTagId(String userTagId, int pageNo, int pageSize) {
        UserTagProductRelation relation = new UserTagProductRelation();
        relation.setTagId(userTagId);
        relation.setStatus(BaseStatusEnum.NORMAL);
        Long total = relationRepository.count(relation);

        Page<UserTagProductRelation> page = new Page(total, pageNo, pageSize);
        // sort
        LinkedHashMap<String, BaseOrderEnum> sort = new LinkedHashMap() { {
            this.put("sort", BaseOrderEnum.ASC);
        }};
        List<UserTagProductRelation> relationList = relationRepository.find(relation, page.getStart(), pageSize, JpaSortUtil.sort(sort));
        if (CollectionUtils.isEmpty(relationList)) {
            return page;
        }
        page.setList(relationList);
        return page;
    }

    public Page<UserTagProductRelation> findPageByPidsUserTagId(String userTagId, List<String> pids, Integer pageNo, Integer pageSize) {
        Page<UserTagProductRelation> page = new Page();
        page.setPageNo(pageNo);
        page.setPageSize(pageSize);

        List<UserTagProductRelation> relationList = relationRepository.findByUserTagIdAndPids(userTagId, pids, page.getStart(), pageSize);
        if (CollectionUtils.isEmpty(relationList)) {
            return page;
        }
        page.setList(relationList);
        page.setTotalSize(relationList.size());
        return page;
    }
}
