package cn.rankin.userservice.service;

import cn.rankin.common.utils.api.page.Page;
import cn.rankin.common.utils.enums.BaseOrderEnum;
import cn.rankin.common.utils.enums.BaseStatusEnum;
import cn.rankin.common.utils.util.JpaSortUtil;
import cn.rankin.data.api.user.dto.UserTagDTO;
import cn.rankin.data.api.user.entity.UserTag;
import cn.rankin.userservice.repository.UserTagRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.LinkedHashMap;
import java.util.List;

import static cn.rankin.userservice.utils.DTOConverter.convert;

@Slf4j
@Service
public class UserTagService {

    @Autowired
    private UserTagRepository userTagRepository;

    public List<UserTag> findByTypeCode(String typeCode,String uid) {
        List<UserTag> userTagList = userTagRepository.findByTypeCodeUid(typeCode,uid, BaseStatusEnum.NORMAL);
        return userTagList;
    }

    public List<UserTag> findByTypeCode(String typeCode) {
        List<UserTag> userTagList = userTagRepository.findByTypeCode(typeCode);
        return userTagList;
    }

    // 搜索用户标签
    public Page<UserTag> search(UserTag userTag, Integer pageNo, Integer pageSize, LinkedHashMap<String, BaseOrderEnum> sort) {
        Long count = userTagRepository.count(userTag);
        Page<UserTag> page = new Page(count, pageNo, pageSize);
        if (count == 0) {
            return page;
        }

        List<UserTag> userTagList = userTagRepository.find(userTag, page.getStart(), page.getPageSize(), JpaSortUtil.sort(sort));
        page.setList(userTagList);

        return page;
    }

    // 标签详情
    public UserTag getUserTag(String id) {
        UserTag userTag = userTagRepository.find(id);
        return userTag;
    }

    // 标签详情
    public List<UserTag> findUserTagByCodeUid(String code,String uid) {
        List<UserTag> userTagLIst = userTagRepository.findByTypeCodeUid(code,uid, BaseStatusEnum.NORMAL);
        return userTagLIst;
    }

    @Transactional
    public UserTag create(UserTagDTO userTagDTO) {
        UserTag userTag = convert(userTagDTO);
        UserTag result = userTagRepository.save(userTag);
        return result;
    }

    @Transactional
    public UserTag update(UserTagDTO userTagDTO) {
        UserTag userTag = convert(userTagDTO);
        UserTag result = userTagRepository.update(userTag);
        return result;
    }

    @Transactional
    public Boolean delete(String tagId) {
        Integer count = userTagRepository.deleteById(tagId);
        if (count > 0) {
            return true;
        }else{
            return false;
        }
    }

    public List<UserTag> findUserTagByUid(String uid) {
        return userTagRepository.findUserTagByUid(uid);
    }

}
